local ADDON, Package = ...

local Module = {}

--------------------------------------------------------------------------------
-- Medusa
--------------------------------------------------------------------------------
local L = Package.L
local is_table = Package.Kernel.is_table
local clone = Package.Enum.clone
local merge = Package.Enum.merge
local purge_keyboard_focus = Package.Utils.purge_keyboard_focus
local create_window = Package.Widget.create_window
local create_tab_button = Package.Widget.create_tab_button
local create_tab_panel = Package.Widget.create_tab_panel
local create_button = Package.Widget.create_button
local create_checkbox = Package.Widget.create_checkbox

--------------------------------------------------------------------------------
-- Predefine
--------------------------------------------------------------------------------
local PREFERENCES = {
  general = {
    {name = "use_item_quality_color_border", module = Package.UseItemQualityColorBorder},
    {name = "use_max_distance", module = Package.UseMaxDistance},
    {name = "use_max_zoom_factor", module = Package.UseMaxZoomFactor},

    {name = "posture_after_action_attempted", module = Package.PostureAfterActionAttempted},
    {name = "repair_when_interact_merchant", module = Package.RepairWhenInteractMerchant},
    {name = "sell_junk_when_interact_merchant", module = Package.SellJunkWhenInteractMerchant},
    {name = "rework_time_alert", module = Package.ReworkTimeAlert},
    {name = "search_profession", module = Package.SearchProfession},

    {name = "indicate_accurate_buff_time", module = Package.IndicateAccurateBuffTime},
    {name = "indicate_item_id_and_level_on_tooltip", module = Package.IndicateItemIDAndLevelOnTooltip},
    {name = "indicate_item_vendor_price_on_tooltip", module = Package.IndicateItemVendorPriceOnTooltip},
    {name = "indicate_target_level_and_class_color_in_trade", module = Package.IndicateTargetLevelAndClassColorInTrade},

    {name = "prevent_unit_combat_feedback", module = Package.PreventUnitCombatFeedback},
    {name = "prevent_ui_error_message_during_combat", module = Package.PreventUIErrorMessageDuringCombat}
  },
  chat = {
    {name = "use_arrow_keys_in_chat", module = Package.UseArrowKeysInChat},
    {name = "increase_chat_history", module = Package.IncreaseChatHistory},
    {name = "unclamp_chat_frame", module = Package.UnclampChatFrame},
    {name = "use_sticky_raid_warning_in_chat", module = Package.UseStickyRaidWarningInChat},
    {name = "use_class_color_in_chat", module = Package.UseClassColorInChat},
    {name = "copy_url_in_chat", module = Package.CopyURLInChat},
    {name = "log_chat", module = Package.LogChat}
  }
}

--------------------------------------------------------------------------------
-- Reform
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- Utility
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- State
--------------------------------------------------------------------------------
local storage = nil

--------------------------------------------------------------------------------
-- Object
--------------------------------------------------------------------------------
local window = nil
local tabs = {}

--------------------------------------------------------------------------------
-- Module
--------------------------------------------------------------------------------
function Module.on_window_drag_start(self) self:GetParent():StartMoving() end

function Module.on_window_drag_stop(self) self:GetParent():StopMovingOrSizing() end

function Module.on_window_mouse_up(self) purge_keyboard_focus() end

function Module.on_tab_select(self)
  Module.select_tab(tabs, self:GetID())
end

function Module.select_tab(tabs, index)
  for at = 1, #tabs do
    tabs[at].button:SetButtonState("NORMAL", false)
    tabs[at].panel:Hide()

    tabs[index].button:SetButtonState("PUSHED", true)
    tabs[index].panel:Show()
  end
end

function Module.on_reload_button_click(self)
  C_UI.Reload()
end

function Module.on_use_checkbox_show(self)
  local preference = PREFERENCES.general[self:GetID()]

  self:SetChecked(storage[preference.name])
end

function Module.on_use_checkbox_click(self)
  local preference = PREFERENCES.general[self:GetID()]

  storage[preference.name] = self:GetChecked()
end

function Module.on_chat_use_checkbox_show(self)
  local preference = PREFERENCES.chat[self:GetID()]

  self:SetChecked(storage[preference.name])
end

function Module.on_chat_use_checkbox_click(self)
  local preference = PREFERENCES.chat[self:GetID()]

  storage[preference.name] = self:GetChecked()
end

function Module.create_configuration_window()
  window = create_window({size = {320, 500}, name = ADDON .. "Config"}, UIParent)
  window:SetPoint("CENTER")
  window:SetScript("OnMouseUp", Module.on_window_mouse_up)
  window:SetScript("OnShow", Module.on_window_show)
  window:SetScript("OnHide", Module.on_window_hide)
  window:SetClampedToScreen(false)
end

function Module.create_configuration_tabs()
  local tabs_attrs = {
    {name = "general", text = L["general"]},
    {name = "chat", text = L["chat"]}
  }

  for at = 1, #tabs_attrs do
    local tab = {}

    tab.name = tabs_attrs[at].name

    tab.button = create_tab_button({text = tabs_attrs[at].text}, window)
    tab.button:SetID(at)
    tab.button:SetPoint("TOPLEFT", (at - 1) * 74, 36)
    tab.button:SetScript("OnDragStart", Module.on_window_drag_start)
    tab.button:SetScript("OnDragStop", Module.on_window_drag_stop)
    tab.button:RegisterForDrag("LeftButton")

    tab.panel = create_tab_panel(nil, window)
    tab.panel:SetID(at)
    tab.panel:SetAllPoints(window)

    tabs[at] = tab
  end

  for at = 1, #tabs do
    tabs[at].button:SetScript("OnClick", Module.on_tab_select)
  end
end

function Module.create_configuration_general_tab()
  local relative = nil
  local parent = tabs[1].panel

  for at = 1, #PREFERENCES.general do
    local preference = PREFERENCES.general[at]

    local use_checkbox = create_checkbox({text = L[preference.name]}, parent)
    use_checkbox:SetID(at)
    use_checkbox:SetScript("OnClick", Module.on_use_checkbox_click)
    use_checkbox:SetScript("OnShow", Module.on_use_checkbox_show)
    use_checkbox.text:SetWidth(parent:GetWidth() - 56)
    use_checkbox.text:SetJustifyH("LEFT")

    if not relative then
      use_checkbox:SetPoint("TOPLEFT", 8, -8)
    else
      use_checkbox:SetPoint("TOP", relative, "BOTTOM")
    end

    relative = use_checkbox
  end
end

function Module.create_configuration_chat_tab()
  local relative = nil
  local parent = tabs[2].panel

  for at = 1, #PREFERENCES.chat do
    local preference = PREFERENCES.chat[at]

    local use_checkbox = create_checkbox({text = L[preference.name]}, parent)
    use_checkbox:SetID(at)
    use_checkbox:SetScript("OnClick", Module.on_chat_use_checkbox_click)
    use_checkbox:SetScript("OnShow", Module.on_chat_use_checkbox_show)
    use_checkbox.text:SetWidth(parent:GetWidth() - 56)
    use_checkbox.text:SetJustifyH("LEFT")

    if not relative then
      use_checkbox:SetPoint("TOPLEFT", 8, -8)
    else
      use_checkbox:SetPoint("TOP", relative, "BOTTOM")
    end

    relative = use_checkbox
  end
end

function Module.run()
  storage = MedusaPreludeStorage

  Module.create_configuration_window()
  Module.create_configuration_tabs()
  Module.create_configuration_general_tab()
  Module.create_configuration_chat_tab()

  Module.select_tab(tabs, 1)

  local reload_button = create_button({text = L["reload_button"]}, window)
  reload_button:SetPoint("BOTTOM", 0, 8)
  reload_button:SetScript("OnClick", Module.on_reload_button_click)
end

--------------------------------------------------------------------------------
-- Export
--------------------------------------------------------------------------------
Package.Config = Module